FollowGoal = {}
FollowGoalMT = { __index = FollowGoal }

setmetatable ( FollowGoal, { __index = Goal } )
goals['follow'] = FollowGoal

function FollowGoal:Create ( position, desiredSpeed, maxForce, spine, path, node, eta )
	local goal = {}
	setmetatable ( goal, FollowGoalMT )

	goal.position = position
	goal.desiredSpeed = desiredSpeed
	goal.maxForce = maxForce
	goal.spine = spine
	goal.path = path
	goal.node = node
	goal.eta = eta

	return goal
end
function FollowGoal:Destroy ( )
	self.position = nil
	self.spine = nil
	self.desiredSpeed = nil
	self.maxForce = nil
	self.path = nil
	self.node = nil
	self.eta = nil
end
function FollowGoal:GetType ( )
    return 'follow'
end
function FollowGoal:Reset ( )
    self.path = nil
    self.node = nil
    self.eta = nil
end
function FollowGoal:Process ( delta, owner )
    if self.path == nil then
        self.path = owner.world.pather:SolveXY ( owner.def.position.x, owner.def.position.y, self.position.x, self.position.y )
        if self.path == nil then
            self:Reset ( )
            return GOAL_FAILED, 0
        end
        self.node = 1
    end

	-- minimum time requred for a complete stop
	local time = owner:GetAccelerationTimeLV ( b2_b2Vec2, self.maxForce )
	-- find the future position
	local fp = owner:GetFuturePosition ( time )

	-- project the future position onto the path
	local target = PointOnPath ( fp, self.path, false )

	if target == nil then
        self:Reset ( )
		return GOAL_FAILED, 0
	end

    -- are we moving off the spine?
	local c = b2Vec2 ( target.x - fp.x, target.y - fp.y )
	local cr = c:GetLength ( )

	if cr > self.spine then
	    -- adjust so that we don't move off the spine of the path
		owner:MoveForce ( delta, target, self.desiredSpeed, self.maxForce, true )
	else
		-- we are on-path
		if self.node == #self.path then
			target = self.path[self.node]
		else
			target = self.path[self.node + 1]
		end

		owner:MoveForce ( delta, target, self.desiredSpeed, self.maxForce, false )
	end

    -- is it taking longer than expected to reach the next node?
    if self.eta then
        self.eta = self.eta - delta
        if self.eta <= 0 then
            -- something must have obstructed our path
            self:Reset ( )
            return GOAL_FAILED, 0
        end
    end

	if self.node == #self.path then
		target = self.path[self.node]
	else
		target = self.path[self.node + 1]
	end
    
	-- condition
	local eta = owner:GetArriveTime ( target, self.desiredSpeed + self.spine / delta )
	if eta < delta then
		-- owner:SetPosition ( target )
		if self.node == #self.path then
            self:Reset ( )
			return GOAL_COMPLETED, eta
		end
		-- move onto the next node
		self.node = self.node + 1

		-- distance to the next node
        local distance = owner.def.position:Distance ( self.path[self.node] )
        -- estimated time of arrival to the next node
        self.eta = distance / self.desiredSpeed * 10
	end

	return GOAL_ACTIVE, delta
end
function FollowGoal:CreateXML ( goal )
    local position = tob2Vec2 ( goal.position )
    local desiredSpeed = tonumber ( goal.desiredSpeed )
    local maxForce = tonumber ( goal.maxForce )
    local spine = tonumber ( goal.spine )
    local p = xml.find ( goal, 'Path' )[1]
    local path
    if p then
        path = toPath ( p )
    end
    local node = tonumber ( goal.node )
    local eta = tonumber ( goal.eta )
	local g = FollowGoal:Create ( position, desiredSpeed, maxForce, spine, path, node, eta )

	return g
end
function FollowGoal:GetXML ( )
	local g = xml.new ( "Goal" )
	g.type = self:GetType ( )
    g.position = b2Vec2tostring ( self.position )
	g.desiredSpeed = self.desiredSpeed
	g.maxForce = self.maxForce
	g.spine = self.spine
    if self.path then
        g.path = Pathtostring ( self.path )
    end
	g.node = self.node
	g.eta = self.eta
	return g
end
