MoveGoal = {}
MoveGoalMT = { __index = MoveGoal }

setmetatable ( MoveGoal, { __index = Goal } )
goals['move'] = MoveGoal

function MoveGoal:Create ( reference, position, speed, transformed )
	local goal = {}
	setmetatable ( goal, MoveGoalMT )

    goal.reference = reference
	goal.position = position
	goal.speed = speed
	goal.transformed = transformed

	return goal
end
function MoveGoal:Destroy ( )
    self.reference = nil
	self.position = nil
	self.speed = nil
	self.transformed = nil
end
function MoveGoal:GetType ( )
    return 'move'
end
function MoveGoal:Reset ( )
    self.transformed = nil
end
function MoveGoal:Process ( delta, owner )
	if self.transformed == nil then
        if self.reference == 'world' then
            self.transformed = self.position
        elseif self.reference == 'self' then
            self.transformed = owner.b2body:GetWorldPoint ( self.position )
        else
            local reference = owner.world:FindBody ( self.reference )
            if reference == nil then
                return GOAL_FAILED, 0
            end
            self.transformed = reference.b2body:GetWorldPoint ( self.position )
        end
    end

	if self.speed == nil then
		owner:SetPosition ( self.transformed )
		self:Reset ( )
		return GOAL_COMPLETED, 0
	end
	local used = owner:Move ( delta, self.transformed, self.speed )

	if used < delta then
        self:Reset ( )
        return GOAL_COMPLETED, used
	end
    return GOAL_ACTIVE, used
end
function MoveGoal:CreateXML ( goal )
    local reference = goal.reference
	local position = tob2Vec2 ( goal.position )
	local speed = tonumber ( goal.speed )
    local transformed = tob2Vec2 ( goal.transformed )
	local g = MoveGoal:Create ( reference, position, speed, transformed )
	return g
end
function MoveGoal:GetXML ( )
	local g = xml.new ( "Goal" )
	g.type = self:GetType ( )
	g.reference = self.reference
	g.position = b2Vec2tostring ( self.position )
	g.speed = self.speed
	g.transformed = b2Vec2tostring ( self.transformed )
	return g
end

