MoveForceGoal = {}
MoveForceGoalMT = { __index = MoveForceGoal }

setmetatable ( MoveForceGoal, { __index = Goal } )
goals['moveForce'] = MoveForceGoal

function MoveForceGoal:Create ( reference, position, speed, maxForce, ease, transformed )
	local goal = {}
	setmetatable ( goal, MoveForceGoalMT )

    goal.reference = reference
	goal.position = position
	goal.speed = speed
	goal.maxForce = maxForce
	goal.ease = ease
	goal.transformed = transformed

	return goal
end
function MoveForceGoal:Destroy ( )
    self.reference = nil
	self.position = nil
	self.speed = nil
	self.maxForce = nil
	self.ease = nil
	self.transformed = nil
end
function MoveForceGoal:GetType ( )
    return 'moveForce'
end
function MoveForceGoal:Reset ( )
    self.transformed = nil
end
function MoveForceGoal:Process ( delta, owner )
	if self.transformed == nil then
        if self.reference == 'world' then
            self.transformed = self.position
        elseif self.reference == 'self' then
            self.transformed = owner.b2body:GetWorldPoint ( self.position )
        else
            local reference = owner.world:FindBody ( self.reference )
            if reference == nil then
                return GOAL_FAILED, 0
            end
            self.transformed = reference.b2body:GetWorldPoint ( self.position )
        end
    end

	local used = owner:MoveForce ( delta, self.transformed, self.speed, self.maxForce, self.ease )
	if used < delta then
        self:Reset ( )
	    return GOAL_COMPLETED, used
	end
    return GOAL_ACTIVE, used
end
function MoveForceGoal:CreateXML ( goal )
    local reference = goal.reference
	local position = tob2Vec2 ( goal.position )
	local speed = tonumber ( goal.speed )
	local maxForce = tonumber ( goal.maxForce )
	local ease = toboolean ( goal.ease )
    local transformed = tob2Vec2 ( goal.transformed )
	local g = MoveForceGoal:Create ( reference, position, speed, maxForce, ease, transformed )
	return g
end
function MoveForceGoal:GetXML ( )
	local g = xml.new ( "Goal" )
	g.type = self:GetType ( )
	g.reference = self.reference
	g.position = b2Vec2tostring ( self.position )
	g.speed = self.speed
	g.maxForce = self.maxForce
	g.ease = self.ease
	g.transformed = b2Vec2tostring ( self.transformed )
	return g
end


