WanderGoal = {}
WanderGoalMT = { __index = WanderGoal }

setmetatable ( WanderGoal, { __index = Goal } )
goals['wander'] = WanderGoal

function WanderGoal:Create ( desiredLV, maxForce, desiredAV, maxTorque, distance, angle, radius, jitter, time, elapsed )
	local goal = {}
	setmetatable ( goal, WanderGoalMT )

	goal.desiredLV = desiredLV
	goal.maxForce = maxForce
	goal.desiredAV = desiredAV
	goal.maxTorque = maxTorque
	goal.distance = distance
	goal.angle = angle
	goal.radius = radius
	goal.jitter = jitter
	goal.time = time
	goal.elapsed = elapsed or 0

	return goal
end
function WanderGoal:Destroy ( )
	self.desiredLV = nil
	self.maxForce = nil
	self.desiredAV = nil
	self.maxTorque = nil
	self.distance = nil
	self.angle = nil
	self.radius = nil
	self.jitter = nil
	self.time = nil
	self.elapsed = nil
end
function WanderGoal:GetType ( )
    return 'wander'
end
function WanderGoal:Process ( delta, owner )
	self.elapsed = self.elapsed + delta
	if self.elapsed >= self.time then
	    local used = delta - ( self.elapsed - self.time )
	    self.elapsed = 0
		return GOAL_COMPLETED, used
	end

	local target = b2Vec2 ( 0, self.radius )
	local displacement = ( 100 - math.random ( 0, 200 ) ) / 100
	self.angle = self.angle + displacement * self.jitter
	target:SetAngle ( self.angle )

	-- project to the front of the body
	target.y = target.y + self.distance
	local w = owner.b2body:GetWorldPoint ( target )

	owner:MoveForce ( delta, w, self.desiredLV, self.maxForce, true )
	owner:FaceTorque ( delta, w, self.desiredAV, self.maxTorque, true )

	return GOAL_ACTIVE, delta
end
function WanderGoal:CreateXML ( goal )
	local desiredLV = tonumber ( goal.desiredLV )
	local maxForce = tonumber ( goal.maxForce )
	local desiredAV = tonumber ( goal.desiredAV )
	local maxTorque = tonumber ( goal.maxTorque )
	local distance = tonumber ( goal.distance )
	local angle = tonumber ( goal.angle )
	local radius = tonumber ( goal.radius )
	local jitter = tonumber ( goal.jitter )
	local time = tonumber ( goal.time )
	local elapsed = tonumber ( goal.elapsed )

	local g = WanderGoal:Create ( desiredLV, maxForce, desiredAV, maxTorque, distance, angle, radius, jitter, time, elapsed )
	return g
end
function WanderGoal:GetXML ( )
	local g = xml.new ( "Goal" )
	g.type = self:GetType ( )
	g.desiredLV = self.desiredLV
	g.maxForce = self.maxForce
	g.desiredAV = self.desiredAV
	g.maxTorque = self.maxTorque
	g.distance = self.distance
	g.angle = self.angle
	g.radius = self.radius
	g.jitter = self.jitter
	g.time = self.time
	g.elapsed = self.elapsed
	return g
end


