-- the parallel goal executes several subgoals simultaneously
-- completes when all of its subgoals are completed successfully
-- fails when one of its subgoals fails

ParallelGoal = {}
ParallelGoalMT = { __index = ParallelGoal }

setmetatable ( ParallelGoal, { __index = Goal } )
goals['parallel'] = ParallelGoal

function ParallelGoal:Create ( loop )
	local goal = {}
	setmetatable ( goal, ParallelGoalMT )

    goal.loop = loop
	goal.goals = {}

	return goal
end

function ParallelGoal:Destroy ( )
	self:Clear ( )

    self.loop = nil
	self.goals = nil
end

function ParallelGoal:GetType ( )
    return 'parallel'
end

function ParallelGoal:Reset ( )
    if self.active ~= nil then
        for i, v in pairs ( self.active ) do
            self.goals[v]:Reset ( )
        end
        self.active = nil
    end
end

function ParallelGoal:Process ( delta, owner )
	return self:ProcessSubGoals ( delta, owner )
end

function ParallelGoal:Message ( owner, type, ... )
    local result = false
	if self.active then
        for i, v in pairs ( self.active ) do
            local active = self.goals[v]
            if active.Message == nil then
            assert ( false, active:GetType ( ) )
            end
            if active:Message ( owner, type, ... ) == true then
                result = true
            end
        end
    end
	return result
end

function ParallelGoal:ProcessSubGoals ( delta, owner )
	if #self.goals == 0 then
		return GOAL_INACTIVE, 0
	end

	-- activate if inactive
    if self.active == nil then
        self.active = {}
        for i in pairs ( self.goals ) do
            table.insert ( self.active, i )
        end
    end

	local maxUsed = 0
	for i, v in pairs ( self.active ) do
	    local active = self.goals[v]

        -- process the current goal
        local status, used = active:Process ( delta, owner )
        maxUsed = math.max ( maxUsed, used )

        if status == GOAL_FAILED then
            -- the failed subgoal is no longer active
            table.remove ( self.active, i )
            -- reset all other active subgoals
            self:Reset ( )
            return GOAL_FAILED, maxUsed
        elseif status == GOAL_COMPLETED then
            if self.loop ~= true then
                table.remove ( self.active, i )
            end
        end
    end

    if #self.active == 0 then
        self.acitve = nil
        return GOAL_COMPLETED, maxUsed
    end

    return GOAL_ACTIVE, maxUsed
end

function ParallelGoal:PushBack ( goal )
	table.insert ( self.goals, goal )
end

function ParallelGoal:Clear ( )
	self:Reset ( )
	while #self.goals > 0 do
		local last = table.remove ( self.goals )
		last:Destroy ( )
	end
end

function ParallelGoal:CreateXML ( goal )
    local loop = toboolean ( goal.loop )

	local g = ParallelGoal:Create ( loop )

	for i = 1, #goal, 1 do
		local sub = Goal:CreateXML ( goal[i] )
		table.insert ( g.goals, sub )
	end

	return g
end

function ParallelGoal:GetXML ( )
	local g = xml.new ( "Goal" )
	g.type = self:GetType ( )
	g.loop = self.loop

	for i, v in pairs ( self.goals ) do
		local sub = v:GetXML ( )
		g:append ( sub )
	end

    if self.active then
        local active = ''
        for i, v in pairs ( self.active ) do
            active = active .. v
            if i < #self.active then
                active = active .. ', '
            end
        end
        g.active = active
    end

	return g
end