--
-- ProbabilitySelector goal
--
ProbabilitySelectorGoal = {}
ProbabilitySelectorGoalMT = { __index = ProbabilitySelectorGoal }

setmetatable ( ProbabilitySelectorGoal, { __index = Goal } )
goals['probabilitySelector'] = ProbabilitySelectorGoal

function ProbabilitySelectorGoal:Create ( )
	local goal = {}
	setmetatable ( goal, ProbabilitySelectorGoalMT )

	goal.goals = {}

	return goal
end

function ProbabilitySelectorGoal:Destroy ( )
	self:Clear ( )

	self.goals = nil
end

function ProbabilitySelectorGoal:GetType ( )
    return 'probabilitySelector'
end

function ProbabilitySelectorGoal:Reset ( )
    if self.active ~= nil then
        self.goals[self.active]:Reset ( )
        self.active = nil
    end
end

function ProbabilitySelectorGoal:Process ( delta, owner )
	return self:ProcessSubGoals ( delta, owner )
end

function ProbabilitySelectorGoal:Message ( owner, type, ... )
	if self.active then
		return self.goals[self.active]:Message ( owner, type, ... )
	end
	return false
end

function ProbabilitySelectorGoal:ProcessSubGoals ( delta, owner )
	if #self.goals == 0 then
		return GOAL_INACTIVE, 0
	end

	-- activate if inactive
    if self.active == nil then
        self.active = math.random ( #self.goals )
    end

	-- process the active goal
	local status, used = self.goals[self.active]:Process ( delta, owner )

    if status == GOAL_ACTIVE then
        return GOAL_ACTIVE, used
    elseif status == GOAL_FAILED then
        -- move to the next goal
        self.active = math.random ( #self.goals )

        return self:Process ( delta - used, owner )
    elseif status == GOAL_COMPLETED then
        -- the selector is complete
        self.active = nil
        return GOAL_COMPLETED, used
    else
        -- unknown goal status returned
        assert ( false, "Unknown goal status returned" )
    end
end

function ProbabilitySelectorGoal:PushBack ( goal )
	table.insert ( self.goals, goal )
end

function ProbabilitySelectorGoal:Clear ( )
	self:Reset ( )
	while #self.goals > 0 do
		local last = table.remove ( self.goals )
		last:Destroy ( )
	end
end

function ProbabilitySelectorGoal:CreateXML ( goal )
	local g = ProbabilitySelectorGoal:Create ( )
	g.active = tonumber ( goal.active )

	for i = 1, #goal, 1 do
		local sub = Goal:CreateXML ( goal[i] )
		table.insert ( g.goals, sub )
	end

	return g
end

function ProbabilitySelectorGoal:GetXML ( )
	local g = xml.new ( "Goal" )
	g.type = self:GetType ( )
	g.active = self.active

	for i, v in pairs ( self.goals ) do
		local sub = v:GetXML ( )
		g:append ( sub )
	end

	return g
end