--
-- Sequence goal
--
SequenceGoal = {}
SequenceGoalMT = { __index = SequenceGoal }

setmetatable ( SequenceGoal, { __index = Goal } )
goals['sequence'] = SequenceGoal

function SequenceGoal:Create ( loop )
	local goal = {}
	setmetatable ( goal, SequenceGoalMT )

	goal.goals = {}
	goal.loop = loop

	return goal
end

function SequenceGoal:Destroy ( )
	self:Clear ( )

	self.goals = nil
	self.loop = nil
end

function SequenceGoal:GetType ( )
    return 'sequence'
end

function SequenceGoal:Reset ( )
    if self.active ~= nil then
        self.goals[self.active]:Reset ( )
        self.active = nil
    end
end

function SequenceGoal:Process ( delta, owner )
	return self:ProcessSubGoals ( delta, owner )
end

function SequenceGoal:Message ( owner, type, ... )
	if self.active then
		return self.goals[self.active]:Message ( owner, type, ... )
	end
	return false
end

function SequenceGoal:ProcessSubGoals ( delta, owner )
	if #self.goals == 0 then
		return GOAL_INACTIVE, 0
	end

	-- activate if inactive
    if self.active == nil then
        self.active = 1
    end

	-- process the active goal
	local status, used = self.goals[self.active]:Process ( delta, owner )

    if status == GOAL_ACTIVE then
        return GOAL_ACTIVE, used
    elseif status == GOAL_FAILED then
        -- the sequence has failed
        self.active = nil
        return GOAL_FAILED, used
    elseif status == GOAL_COMPLETED then
        -- move to the next goal
        self.active = self.active + 1
        if self.active > #self.goals then
            if self.loop ~= true then
                -- the sequence is complete
                self.active = nil
                return GOAL_COMPLETED, used
            end
            self.active = 1
        end
        if delta == used then
            return GOAL_ACTIVE, used
        end
        return self:Process ( delta - used, owner )
    else
        -- unknown goal status returned
        assert ( false, "Unknown goal status returned" )
    end
end

function SequenceGoal:PushBack ( goal )
	table.insert ( self.goals, goal )
end

function SequenceGoal:Clear ( )
	self:Reset ( )
	while #self.goals > 0 do
		local last = table.remove ( self.goals )
		last:Destroy ( )
	end
end

function SequenceGoal:CreateXML ( goal )
	local loop = toboolean ( goal.loop )
	local g = SequenceGoal:Create ( loop )
	g.active = tonumber ( goal.active )

	for i = 1, #goal, 1 do
		local sub = Goal:CreateXML ( goal[i] )
		table.insert ( g.goals, sub )
	end

	return g
end

function SequenceGoal:GetXML ( )
	local g = xml.new ( "Goal" )
	g.type = self:GetType ( )
	g.active = self.active
	g.loop = self.loop

	for i, v in pairs ( self.goals ) do
		local sub = v:GetXML ( )
		g:append ( sub )
	end

	return g
end