--
-- b2Vec2
--
b2_b2Vec2 = b2Vec2 ( 0, 0 )

-- changes the current angle of the vector in radians
function b2Vec2:ChangeAngle ( r )
	assert ( ( self.x == 0 and self.y == 0 ) == false, "b2Vec2 is zero length" )

	local cos_d = math.cos ( r )
	local sin_d = math.sin ( r )

	local tx = cos_d * self.x - sin_d * self.y
	local ty = cos_d * self.y + sin_d * self.x

	self.x = tx
	self.y = ty
end

-- returns the angle of the vector in radians
-- west = -1.57, north = 0, east = 1.57, south = 3.14
function b2Vec2:GetAngle ( )
	assert ( ( self.x == 0 and self.y == 0 ) == false, "b2Vec2 is zero length" )

	return math.atan2 ( self.x, self.y )
end

-- sets the angle of the vector in radians
-- west = -1.57, north = 0, east = 1.57, south = 3.14
function b2Vec2:SetAngle ( new )
	local old = self:GetAngle ( )

	self:ChangeAngle ( new - old )
end

-- sets the length of the vector
function b2Vec2:SetLength ( n )
	self:Normalize ( )

	self.x = self.x * n
	self.y = self.y * n
end

-- truncates the length of the vector
function b2Vec2:Truncate ( length )
	local dist = self:Length ( )

	if dist > length then
		self:SetLength ( length )
	end
end

-- returns the squared distance between two points
function b2Vec2:DistanceSquared ( pt )
	local dx = pt.x - self.x
	local dy = pt.y - self.y

	return dx * dx + dy * dy
end

-- returns the distance between two points
function b2Vec2:Distance ( pt )
	local dist = self:DistanceSquared ( pt )

	return math.sqrt ( dist )
end

-- returns the angle between two points in radians
-- west = -1.57, north = 0, east = 1.57, south = 3.14
function b2Vec2:Direction ( pt )
	return math.atan2 ( pt.x - self.x, pt.y - self.y )
end

-- returns the dot product of two points
-- negative when the angle between the two vectors is greater than 90
-- positive when the angle between the two vectors is less than 90
-- zero when the two vectors are perpendicular
function b2Vec2:Dot ( pt )
	return self.x * pt.x + self.y * pt.y
end

function b2Vec2:Arc ( pt )
    local dot = self:Dot ( pt )
    local l1 = self:Length ( )
    local l2 = self:Length ( )
    return math.acos ( dot / ( l1 * l2 ) )
end

