--
-- Joint
--
Joint = {}
JointMT = { __index = Joint }

function Joint:Destroy ( )
	self:Degenerate ( )

	self.def = nil
	self.body1 = nil
	self.body2 = nil
	self.world = nil
end

function Joint:GetBody1 ( )
	return self.body1
end

function Joint:GetBody2 ( )
	return self.body2
end

function Joint:Generate ( )
	self:Degenerate ( )

	self.def.body1 = self.body1.b2body
	self.def.body2 = self.body2.b2body

	if self.def.body1 == nil or self.def.body2 == nil then
		return false
	end

	self.b2joint = self.world.b2world:CreateJoint ( self.def )
	self.b2joint.parent = self

	return true
end

function Joint:Degenerate ( )
	if self.b2joint == nil then
		return
	end

	self.b2joint.parent = nil
	self.world.b2world:DestroyJoint ( self.b2joint )
	self.b2joint = nil
end


RevoluteJoint = {}
RevoluteJointMT = { __index = RevoluteJoint }

setmetatable ( RevoluteJoint, { __index = Joint } )

function RevoluteJoint:Create ( world, body1, body2, collideConnected, a )
	local joint = {}
	setmetatable ( joint, RevoluteJointMT )

	joint.def = b2RevoluteJointDef ( )
	joint.def:Initialize ( body1.b2body, body2.b2body, a )
	joint.def.collideConnected = collideConnected

	joint.body1 = body1
	joint.body2 = body2
	joint.world = world

	if joint:Generate ( ) == false then
		return nil
	end

	return joint
end

function RevoluteJoint:GetType ( )
	return 'revolute'
end

function RevoluteJoint:SetMotor ( enableMotor, motorSpeed, maxMotorTorque )
	self.def.enableMotor = enableMotor
	self.def.motorSpeed = motorSpeed
	self.def.maxMotorTorque = maxMotorTorque

	self.b2joint:EnableMotor ( enableMotor )
	self.b2joint:SetMotorSpeed ( motorSpeed )
	self.b2joint:SetMaxMotorTorque ( maxMotorTorque )
end

function RevoluteJoint:SetLimit ( enableLimit, lowerAngle, upperAngle )
	self.def.enableLimit = enableLimit
	self.def.lowerAngle = lowerAngle
	self.def.upperAngle = upperAngle

	self.b2joint:EnableLimit ( enableLimit )
	self.b2joint:SetLimits ( lowerAngle, upperAngle )
end




PrismaticJoint = {}
PrismaticJointMT = { __index = PrismaticJoint }

setmetatable ( PrismaticJoint, { __index = Joint } )

function PrismaticJoint:Create ( world, body1, body2, collideConnected, a, axis )
	local joint = {}
	setmetatable ( joint, PrismaticJointMT )

	joint.def = b2PrismaticJointDef ( )
	joint.def:Initialize ( body1.b2body, body2.b2body, a, axis )
	joint.def.collideConnected = collideConnected

	joint.body1 = body1
	joint.body2 = body2
	joint.world = world

	if joint:Generate ( ) == false then
		return nil
	end

	return joint
end

function PrismaticJoint:GetType ( )
	return 'prismatic'
end

function PrismaticJoint:SetMotor ( enableMotor, motorSpeed, maxMotorForce )
	self.def.enableMotor = enableMotor
	self.def.motorSpeed = motorSpeed
	self.def.maxMotorForce = maxMotorForce

	self.b2joint:EnableMotor ( enableMotor )
	self.b2joint:SetMotorSpeed ( motorSpeed )
	self.b2joint:SetMaxMotorForce ( maxMotorForce )
end

function PrismaticJoint:SetLimit ( enableLimit, lowerTranslation, upperTranslation )
	self.def.enableLimit = enableLimit
	self.def.lowerTranslation = lowerTranslation
	self.def.upperTranslation = upperTranslation

	self.b2joint:EnableLimit ( enableLimit )
	self.b2joint:SetLimits ( lowerTranslation, upperTranslation )
end


PulleyJoint = {}
PulleyJointMT = { __index = PulleyJoint }

setmetatable ( PulleyJoint, { __index = Joint } )

function PulleyJoint:Create ( world, body1, body2, collideConnected, ga1, ga2, a1, a2, ratio )
	local joint = {}
	setmetatable ( joint, PulleyJointMT )

	joint.def = b2PulleyJointDef ( )
	joint.def:Initialize ( body1.b2body, body2.b2body, ga1, ga2, a1, a2, ratio )
	joint.def.collideConnected = collideConnected

	joint.body1 = body1
	joint.body2 = body2
	joint.world = world

	if joint:Generate ( ) == false then
		return nil
	end

	return joint
end

function PulleyJoint:GetType ( )
	return 'pulley'
end


DistanceJoint = {}
DistanceJointMT = { __index = DistanceJoint }

setmetatable ( DistanceJoint, { __index = Joint } )

function DistanceJoint:Create ( world, body1, body2, collideConnected, a1, a2 )
	local joint = {}
	setmetatable ( joint, DistanceJointMT )

	joint.def = b2DistanceJointDef ( )
	joint.def:Initialize ( body1.b2body, body2.b2body, a1, a2 )
	joint.def.collideConnected = collideConnected

	joint.body1 = body1
	joint.body2 = body2
	joint.world = world

	if joint:Generate ( ) == false then
		return nil
	end

	return joint
end

function DistanceJoint:GetType ( )
	return 'distance'
end

--
-- XML
--
function Joint:GetXML ( )
	local joint = xml.new ( "Joint" )

	local b1 = self:GetBody1 ( )
	local b2 = self:GetBody2 ( )

	local type = self:GetType ( )

	joint.type = type
	joint.body1 = b1.name
	joint.body2 = b2.name
	joint.collideConnected = self.def.collideConnected

	if type == 'distance' then
		local a1 = self.b2joint:GetAnchor1 ( )
		local a2 = self.b2joint:GetAnchor2 ( )

		joint:append ( "Anchor1" )[1] = b2Vec2tostring ( a1 )
		joint:append ( "Anchor2" )[1] = b2Vec2tostring ( a2 )

		joint:append ( "FrequencyHz" )[1] = self.def.frequencyHz
		joint:append ( "DampingRatio" )[1] = self.def.dampingRatio
	elseif type == 'revolute' then
		local a = self.b2joint:GetAnchor1 ( )

		joint:append ( "Anchor1" )[1] = b2Vec2tostring ( a )

		local limit = joint:append ( "Limit" )
		limit.enabled = self.def.enableLimit
		limit.lowerAngle = self.def.lowerAngle
		limit.upperAngle = self.def.upperAngle

		local motor = joint:append ( "Motor" )
		motor.enabled = self.def.enableMotor
		motor.speed = self.def.motorSpeed
		motor.maxTorque = self.def.maxMotorTorque
	elseif type == 'pulley' then
		local a1 = v.b2joint:GetAnchor1 ( )
		local a2 = v.b2joint:GetAnchor2 ( )
		local ga1 = v.b2joint:GetGroundAnchor1 ( )
		local ga2 = v.b2joint:GetGroundAnchor2 ( )

		joint:append ( "Anchor1" )[1] = b2Vec2tostring ( a1 )
		joint:append ( "Anchor2" )[1] = b2Vec2tostring ( a2 )
		joint:append ( "Ground1" )[1] = b2Vec2tostring ( ga1 )
		joint:append ( "Ground2" )[1] = b2Vec2tostring ( ga2 )
	elseif type == 'prismatic' then
		local a = self.b2joint:GetAnchor1 ( )
		-- convert the local axis to world coordinates
		local axis = self.body1.b2body:GetWorldVector ( self.def.localAxis1 )

		joint:append ( "Anchor1" )[1] = b2Vec2tostring ( a )
		joint:append ( "Axis" )[1] = b2Vec2tostring ( axis )

		local limit = joint:append ( "Limit" )
		limit.enabled = self.def.enableLimit
		limit.lowerTranslation = self.def.lowerTranslation
		limit.upperTranslation = self.def.upperTranslation

		local motor = joint:append ( "Motor" )
		motor.enabled = self.def.enableMotor
		motor.speed = self.def.motorSpeed
		motor.maxForce = self.def.maxMotorForce
	end

	return joint
end
