MovePlatformGoal = {}
MovePlatformGoalMT = { __index = MovePlatformGoal }

setmetatable ( MovePlatformGoal, { __index = Goal } )
goals['movePlatform'] = MovePlatformGoal

function MovePlatformGoal:Create ( reference, position, speed, transformed )
	local goal = {}
	setmetatable ( goal, MovePlatformGoalMT )

    goal.reference = reference
	goal.position = position
	goal.speed = speed
	goal.transformed = transformed

	return goal
end
function MovePlatformGoal:Destroy ( )
    self.reference = nil
	self.position = nil
	self.speed = nil
	self.transformed = nil
end
function MovePlatformGoal:GetType ( )
    return 'movePlatform'
end
function MovePlatformGoal:Reset ( )
    self.transformed = nil
end
function MovePlatformGoal:Process ( delta, owner )
	if self.transformed == nil then
        if self.reference == 'world' then
            self.transformed = self.position
        elseif self.reference == 'self' then
            self.transformed = owner.b2body:GetWorldPoint ( self.position )
        else
            local reference = owner.world:FindBody ( self.reference )
            if reference == nil then
                return GOAL_FAILED, 0
            end
            self.transformed = reference.b2body:GetWorldPoint ( self.position )
        end
    end

    local old = b2Vec2 ( owner.def.position )

	if self.speed == nil then
		owner:SetPosition ( self.transformed )
		self:Reset ( )
		return GOAL_COMPLETED, 0
	end
	local used = owner:Move ( delta, self.transformed, self.speed )

    local new = b2Vec2 ( owner.def.position )
    for i, v in pairs ( owner.collisions ) do
        local p = owner.world:FindBody ( v[1] )
        if p then
            if p.name == 'player' or p.name == 'player2' then
                local d = b2Vec2 ( new.x - old.x, new.y - old.y )
                p:SetPositionXY ( p.def.position.x + d.x, p.def.position.y + d.y )
                break
            end
        end
    end

	if used < delta then
        self:Reset ( )
        return GOAL_COMPLETED, used
	end
    return GOAL_ACTIVE, used
end
function MovePlatformGoal:CreateXML ( goal )
    local reference = goal.reference
	local position = tob2Vec2 ( goal.position )
	local speed = tonumber ( goal.speed )
    local transformed = tob2Vec2 ( goal.transformed )
	local g = MovePlatformGoal:Create ( reference, position, speed, transformed )
	return g
end
function MovePlatformGoal:GetXML ( )
	local g = xml.new ( "Goal" )
	g.type = self:GetType ( )
	g.reference = self.reference
	g.position = b2Vec2tostring ( self.position )
	g.speed = self.speed
	g.transformed = b2Vec2tostring ( self.transformed )
	return g
end

