
function SignedTriangleArea ( a, b, c )
	return ( a.x - c.x ) * ( b.y - c.y ) - ( a.y - c.y ) * ( b.x - c.x )
end

function PolygonArea ( vertices )
	local area = 0
	local points = {}
	local ret = b2Fill ( vertices, points )
	for i = 1, #points, 3 do
	    local sta = SignedTriangleArea ( points[i], points[i + 1], points[i + 2] )
		area = area + math.abs ( sta )
	end
	return area
end

function VectorIntersect ( sa1, sb1, sa2, sb2 )
	local a1 = SignedTriangleArea ( sa1, sb1, sb2 )
	local a2 = SignedTriangleArea ( sa1, sb1, sa2 )

	if ( a1 * a2 < 0.0 ) then
		local a3 = SignedTriangleArea ( sa2, sb2, sa1 )
		local a4 = a3 + a2 - a1
		if ( a3 * a4 < 0.0 ) then
			local t = a3 / ( a3 - a4 )
			local ix = sa1.x + t * ( sb1.x - sa1.x )
			local iy = sa1.y + t * ( sb1.y - sa1.y )
			return b2Vec2 ( ix, iy )
		end
	end

	return nil
end

function VectorPolygonIntersect ( vertices, a, b )
	local intersects = {}
	local indexes = {}

	for i, current in pairs ( vertices ) do
		local next
		if i == #vertices then
			next = vertices[1]
		else
			next = vertices[i + 1]
		end

		local pt = VectorIntersect ( a, b, current, next )
		if pt ~= nil then
			table.insert ( intersects, pt )
			table.insert ( indexes, i )
		end
	end

	return intersects, indexes
end

function PointOnLine ( pt, a, b, isSegment )
	local norm_ab = b2Vec2 ( b.x - a.x, b.y - a.y )
	local norm_pt = b2Vec2 ( pt.x - a.x, pt.y - a.y )

	local d1 = norm_pt:Dot ( norm_ab )
	local d2 = norm_ab:Dot ( norm_ab )

	assert ( d2 ~= 0 )

	local u = d1 / d2

	if isSegment == true then
		if u < 0.0 then
			u = 0
		else
			if u > 1.0 then
				u = 1.0
			end
		end
	end

	return b2Vec2 ( a.x + u * norm_ab.x, a.y + u * norm_ab.y )
end

function PointOnPath ( pt, path, closed )
	local res = nil
	local res_distance = nil

	for i = 1, #path, 1 do
		local a = path[i]
		local b = path[i + 1]
		if i == #path then
			if closed == true then
				b = path[1]
			else
				break
			end
		end
		local r = PointOnLine ( pt, a, b, true )
		local r_distance = r:Distance ( pt )
		if res == nil or res_distance > r_distance then
			res = r
			res_distance = r_distance
		end
	end

	return res
end
