function LoadLine ( level, map, dest, x )
    for y = 1, #map, 1 do
        local v = map[y][x]
        local p = b2Vec2 ( x * 3.2, ( #map - y ) * 3.2 )

        dest[y][x] = level:CreateTileBody ( v, p )
    end
end

function UnLoadLine ( level, map, dest, x )
    for y = 1, #map, 1 do
        if dest[y][x] then
            level.world:DestroyBody ( dest[y][x] )
            dest[y][x] = nil
        end
    end
end


function ShapePolygon:GetMinY ( )
    local y = self.vertices[1].y
    for i = 2, #self.vertices, 1 do
        y = math.min ( y, self.vertices[i].y )
    end
    return y
end
function ShapePolygon:GetMaxY ( )
    local y = self.vertices[1].y
    for i = 2, #self.vertices, 1 do
        y = math.max ( y, self.vertices[i].y )
    end
    return y
end
function ShapeCircle:GetMinY ( )
    return self.def.localPosition.y - self.def.radius
end
function ShapeCircle:GetMaxY ( )
    return self.def.localPosition.y + self.def.radius
end
function Body:GetMinY ( )
    local y = self.shapes[1]:GetMinY ( )
    for i = 2, #self.shapes, 1 do
        y = math.min ( y, self.shapes[i]:GetMinY ( ) )
    end
    return self.def.position.y + y
    --return self.b2body:GetWorldPoint ( b2Vec2 ( 0, y ) ).y
end
function Body:GetMaxY ( )
    local y = self.shapes[1]:GetMaxY ( )
    for i = 2, #self.shapes, 1 do
        y = math.max ( y, self.shapes[i]:GetMaxY ( ) )
    end
    return self.def.position.y + y
    --return self.b2body:GetWorldPoint ( b2Vec2 ( 0, y ) ).y
end

function PlayerVsBody ( p, b2 )
    if #p.shapes == 0 or #b2.shapes == 0 then
        return true
    end
    if b2.isPlatform == true then
        local py = p:GetMinY ( )
        local b2y = b2:GetMaxY ( )
        local lv = p.b2body:GetLinearVelocity ( ).y * ( game.timer.interval / 1000 )
        if lv < 0 then
            py = py + math.abs ( lv )
        end
        local d = py - b2y
        if d < -0.01 then
            return false
        end
    end
    return true
end

cf = b2ContactFilter ( )
cf.ShouldCollide = function ( cf, s1, s2 )
    local p1 = s1.parent
    local p2 = s2.parent
    if p1 == nil or p2 == nil then
        return true
    end
    local b1 = p1.body
    local b2 = p2.body
    if b1.agent == true and b2.agent == true then
        return false
    end
    if b1.passThrough == true then
        return PlayerVsBody ( b1, b2 )
    end
    if b2.passThrough == true then
        return PlayerVsBody ( b2, b1 )
    end
    return true
end